from scipy import misc, ndimage
import matplotlib.pyplot as plt
import numpy as np

def angle_P(ideal_angle):
	#Wolkenkamera_FE8174V 4.1 (Korrektur des Höhenwinkels)
	deg = np.rad2deg(ideal_angle)
	ideal_deg = np.polynomial.polynomial.polyval(deg, [2.494295303, 1.326190211, -0.01122405179, 0.0001384399783, -0.0000006380024219])
	return np.deg2rad(ideal_deg)

def angle_Q(real_angle):
	#Wolkenkamera_FE8174V 4.1 (Korrektur des Höhenwinkels)
	deg = np.rad2deg(real_angle)
	real_deg = np.polynomial.polynomial.polyval(deg, [-1.817333483 ,0.6890441144 ,0.008969454015, -0.00009844546547, 0.0000004407488186])
	return np.deg2rad(real_deg)

def to_hemishere(rx, ry):
	#rx, ry normalized radius of source image coords
	r = np.sqrt(np.square(rx) + np.square(ry))
	alpha = np.arctan2(rx,ry)
	epsilon = np.pi/2 * (1 - r)
	return (alpha, epsilon)

def to_cloud_plane(epsilon, H):
	r_real = (1 - epsilon / (np.pi/2))
	eps = np.pi/2 - np.arctan(r_real / H)
	#map to cloud plane with height H
	r_proj = 1 - eps / (np.pi/2)
	return r_proj

def output_coordinate(alpha, radius, size):
	rxp = np.sin(alpha) * r_proj
	ryp = np.cos(alpha) * r_proj
	#source coordinates of input image
	return np.around([(rxp + 1)*size/2, (ryp + 1)*size/2]).astype(np.int)

face = misc.imread('cloudimage.jpg')
#face = misc.face(gray=True)

grayFace = face.sum(axis=2) / 3

w,h, cc = face.shape

H = 0.5

pW, pH = (w, h)

X, Y = np.ogrid[0:pW, 0:pH]

#xy distance from center of projection rx,ry in [-1,1]
rx = X/(pW/2) - 1
ry = Y/(pH/2) - 1


alpha, epsilon = to_hemishere(rx, ry)
#angle correction
epsilon_real = angle_P(epsilon)
#epsilon_real = epsilon

r_proj = to_cloud_plane(epsilon_real, H)

#ouput image coordiates
x_img, y_img = output_coordinate(alpha, r_proj, w)


print(x_img, y_img)
projImage = face[x_img, y_img]

print('InputImage:', grayFace.shape, grayFace.dtype)
print('projImage:', projImage.shape, projImage.dtype)
plt.subplot(121)
plt.imshow(grayFace, cmap=plt.cm.gray, interpolation='nearest')
plt.axis('off')

plt.subplot(122)
plt.imshow(projImage, cmap=plt.cm.gray, interpolation='nearest')
plt.axis('off')
plt.show()
