from cloudproc.models import ConvnetVGG16
from cloudproc.config import SUMMARIES_DIR, BATCH_SIZE, HEIGHT_BINS, DATASET_DIR, VALIDATION_FILE
from cloudproc.cloud_cnn import _parse_function, heights_to_labels, num_height_classes
import tensorflow as tf
import numpy as np
import cv2
import os


def min_max_norm(batch_imgs):
    batch_imgs = tf.transpose(batch_imgs)
    min_value = tf.reduce_min(tf.reduce_min(tf.reduce_min(batch_imgs, axis=0), axis=0), axis=0)
    max_value = tf.reduce_max(tf.reduce_max(tf.reduce_max(batch_imgs, axis=0), axis=0), axis=0)
    print(min_value.shape, max_value.shape)
    normal_imgs = 255 * (batch_imgs - min_value) / (max_value - min_value)
    print(normal_imgs.shape)
    return tf.transpose(normal_imgs)

def visualise_VGG16_activations(checkpoint_dir=SUMMARIES_DIR, batch_size=BATCH_SIZE):
    validation_filename = os.path.join(DATASET_DIR, VALIDATION_FILE)
    val_dataset = tf.data.TFRecordDataset(validation_filename).map(
        lambda x: _parse_function(x), num_parallel_calls=2)
    val_dataset = val_dataset.batch(batch_size)
    validation_iterator = val_dataset.make_one_shot_iterator()

    print('Batch size:', batch_size)

    images_big, height_readings, rain_quantity, sun_insolation = validation_iterator.get_next()
    labels = heights_to_labels(height_readings)
    # resize for vgg16 (224x224)
    images = tf.image.resize_images(images_big, (224,224))

    conv_net = ConvnetVGG16(images, classes=num_height_classes())
    keep_prob = conv_net.keep_prob

    height_classes = conv_net.class_predictions()
    print(conv_net.conv_means.keys())
    def mean_channels(prefix):
        for num in range(1,4):
            conv_name = 'conv{}_{}'.format(prefix, num)
            if conv_name in conv_net.conv_means:
                yield min_max_norm(conv_net.conv_means[conv_name])
            else:
                yield tf.zeros(tf.shape(conv_net.conv_means['conv{}_{}'.format(prefix, 1)]))


    conv_rgb_imgs = [tf.image.resize_nearest_neighbor(tf.concat(
        list(mean_channels(super_layer))
        , axis=-1), (224,224))
                     for super_layer in range(1,6)]

    # row: input_images, conv_rgb_img1, ..., conv_rgb_img5
    conv_rgb_rows = tf.concat([images] + conv_rgb_imgs, axis=-2)
    saver = tf.train.Saver()

    with tf.Session() as sess:
        ckpt = tf.train.get_checkpoint_state(checkpoint_dir)

        if ckpt and ckpt.model_checkpoint_path:
            print('restoring model from:', ckpt.model_checkpoint_path)
            saver.restore(sess, ckpt.model_checkpoint_path)
        else:
            raise ValueError('Could not load checkpoint from {}.'.format(checkpoint_dir))
        class_rows = {}
        while True:
            try:
                h_preds, h_class, imgs_rows = \
                     sess.run([ height_classes, labels,
                                   conv_rgb_rows], feed_dict={keep_prob: 1.0})

                msk = h_preds == h_class
                for c, imgs_row in zip(h_class, imgs_rows):
                    class_rows[c] = imgs_row
                    print('found height_class {} row {}'.format(c, imgs_row.shape))

                if all(k in class_rows for k in range(len(HEIGHT_BINS))):
                    print('found example for each height_class')
                    break

            except (tf.errors.OutOfRangeError, KeyboardInterrupt):
                print('Done searching model.')
                break

    merged_img = np.concatenate([class_rows[c] for c in sorted(class_rows)])
    return merged_img

merged_img = visualise_VGG16_activations()
bgr_img = cv2.cvtColor(merged_img, cv2.COLOR_RGB2BGR)
cv2.imwrite('vis_featmaps.png', bgr_img)
